
/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 *
 * @author n.hoffmann
 * @version $Id: $
 */

package eu.etaxonomy.taxeditor.datasource;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.IPath;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.XMLMemento;

import eu.etaxonomy.cdm.database.CdmDataSource;
import eu.etaxonomy.cdm.database.CdmPersistentDataSource;
import eu.etaxonomy.cdm.database.DataSourceNotFoundException;
import eu.etaxonomy.cdm.database.ICdmDataSource;
import eu.etaxonomy.cdm.model.name.NomenclaturalCode;
import eu.etaxonomy.taxeditor.model.MementoHelper;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.store.internal.TaxeditorStorePlugin;
public class CdmDataSourceRepository{;

	/** Constant <code>TAG_DATASOURCE="tagDataSource"</code> */
	public static final String TAG_DATASOURCE = "tagDataSource";
	private static final String CURRENT_DATASOURCE = "currentDataSource";
	private static final String DEFAULT_DATASOURCE_STATE_FILENAME = "datasource.xml";
	private static final String DEFAULT_DATASOURCE_NAME = "cdm";
	
	private static ICdmDataSource currentDataSource;
	private static IMemento memento;
	private static String lastUsedDataSourceName;
	
	/**
	 * <p>Getter for the field <code>lastUsedDataSourceName</code>.</p>
	 *
	 * @return a {@link java.lang.String} object.
	 */
	public static String getLastUsedDataSourceName(){
		if(lastUsedDataSourceName == null){
			memento = readMemento();		
			lastUsedDataSourceName = memento != null ? memento.getString(CURRENT_DATASOURCE) : DEFAULT_DATASOURCE_NAME;
		}
		return lastUsedDataSourceName;
	}

	/**
	 * <p>delete</p>
	 *
	 * @param dataSource a {@link eu.etaxonomy.cdm.database.CdmPersistentDataSource} object.
	 * @return a boolean.
	 */
	public static boolean delete(CdmPersistentDataSource dataSource) {
		CdmPersistentDataSource.delete(dataSource);
		return true;
	}

	
	/**
	 * <p>getAll</p>
	 *
	 * @return a {@link java.util.List} object.
	 */
	public static List<ICdmDataSource> getAll() {
		List<ICdmDataSource> dataSources = new ArrayList<ICdmDataSource>();
		
		for(ICdmDataSource dataSource : CdmPersistentDataSource.getAllDataSources()){
			try {
				dataSources.add(CdmPersistentDataSource.NewInstance(dataSource.getName()));
			} catch (DataSourceNotFoundException e) {
				StoreUtil.error(CdmDataSourceRepository.class, "Could not find dataSource", e);
			}
		}
		
		// TODO sort by database name
		
		return dataSources;
	}
	
	/**
	 * <p>getDataSource</p>
	 *
	 * @param name a {@link java.lang.String} object.
	 * @return a {@link eu.etaxonomy.cdm.database.ICdmDataSource} object.
	 */
	public static ICdmDataSource getDataSource(String name){
		
		for(ICdmDataSource dataSource : CdmPersistentDataSource.getAllDataSources()){
			try {
				if(dataSource.getName().equals(name)){
					return CdmPersistentDataSource.NewInstance(dataSource.getName());
				}
			} catch (DataSourceNotFoundException e) {
				StoreUtil.error(CdmDataSourceRepository.class, "Could not find dataSource", e);
			}
		}
		
		return null;
	}
		
	/**
	 * <p>Getter for the field <code>currentDataSource</code>.</p>
	 *
	 * @return a {@link eu.etaxonomy.cdm.database.ICdmDataSource} object.
	 */
	public static ICdmDataSource getCurrentDataSource() {
		if (currentDataSource == null) {
			try {
				currentDataSource = CdmPersistentDataSource.NewInstance(getLastUsedDataSourceName());
			} catch (DataSourceNotFoundException e) {
				// fallback creates a new default
				ICdmDataSource h2DataSource = CdmDataSource.NewH2EmbeddedInstance(
						DEFAULT_DATASOURCE_NAME, "sa", "", PreferencesUtil.getPreferredNomenclaturalCode());
				save(h2DataSource.getName(), h2DataSource);
				setCurrentDataSource(h2DataSource);
			}
		}
		return currentDataSource;
	}
	
	/**
	 * <p>Setter for the field <code>currentDataSource</code>.</p>
	 *
	 * @param dataSource a {@link eu.etaxonomy.cdm.database.ICdmDataSource} object.
	 * @return a boolean.
	 */
	public static boolean setCurrentDataSource(ICdmDataSource dataSource) {
		currentDataSource = dataSource;
		NomenclaturalCode dataSourceNomenclaturalCode = dataSource.getNomenclaturalCode();
		NomenclaturalCode applicationNomenclaturalCode = PreferencesUtil.getPreferredNomenclaturalCode();
		
		if( dataSourceNomenclaturalCode != null && ! dataSourceNomenclaturalCode.equals(applicationNomenclaturalCode)){
			PreferencesUtil.setPreferredNomenclaturalCode(dataSourceNomenclaturalCode);
			StoreUtil.informationDialog("Nomenclatural Code Change", "The Datasource that was just " +
					"loaded has a different nomenclatural code than the one stored in Preferences. " +
					"The nomenclatural code was changed in the application.");
		}
		return true;
	}	
		
	/**
	 * <p>changeDataSource</p>
	 *
	 * @param dataSource a {@link eu.etaxonomy.cdm.database.ICdmDataSource} object.
	 * @return a boolean.
	 */
	public static boolean changeDataSource(final ICdmDataSource dataSource) {
		saveAsCurrentDatabaseToMemento(dataSource);
		
		CdmStore.connect(dataSource);
		
		return true;
	}

	/**
	 * <p>save</p>
	 *
	 * @param dataSource a {@link eu.etaxonomy.cdm.database.ICdmDataSource} object.
	 * @param strDataSourceName a {@link java.lang.String} object.
	 * @return a {@link eu.etaxonomy.cdm.database.CdmPersistentDataSource} object.
	 */
	public static CdmPersistentDataSource save(String strDataSourceName, ICdmDataSource dataSource) {
		return CdmPersistentDataSource.save(strDataSourceName, dataSource);
	}
	
	/**
	 * <p>update</p>
	 *
	 * @param strDataSourceName a {@link java.lang.String} object.
	 * @param dataSource a {@link eu.etaxonomy.cdm.database.ICdmDataSource} object.
	 * @return a {@link eu.etaxonomy.cdm.database.CdmPersistentDataSource} object.
	 */
	public static CdmPersistentDataSource update(String strDataSourceName, ICdmDataSource dataSource){
		try {
			return CdmPersistentDataSource.update(strDataSourceName, dataSource);
		} catch (Exception e) {
			StoreUtil.error(CdmDataSourceRepository.class, "Error updating datasource", e);
		}
		return null;
	}
	
	/*********************************************************
	 * Memento Handling										 *
	 *********************************************************/
	
	private static void saveAsCurrentDatabaseToMemento(ICdmDataSource dataSource){
		if(memento == null)
			memento = readMemento();
		memento.putString(CURRENT_DATASOURCE, dataSource.getName());
		saveMementoToFile(memento);
	}
	
	/*
	 * Answer the workbench state file.
	 */
	private static File getDataSourceStateFile() {
		IPath path = TaxeditorStorePlugin.getDefault().getStateLocation();
		if (path == null) {
			return null;
		}
		path = path.append(DEFAULT_DATASOURCE_STATE_FILENAME);
		return path.toFile();
	}
	
	private static IMemento readMemento(){
		try {
			return MementoHelper.readMementoFromFile(getDataSourceStateFile());
		} catch (FileNotFoundException e) {
			return initializeMemento();
		}
	}
	

	private static IMemento initializeMemento() {
		
		XMLMemento memento = XMLMemento.createWriteRoot(TAG_DATASOURCE);
		memento.putString(CURRENT_DATASOURCE, DEFAULT_DATASOURCE_NAME);
		saveMementoToFile(memento);
		
		return readMemento();
	}

	/*
	 * Save the workbench UI in a persistence file.
	 */
	private static IMemento saveMementoToFile(IMemento memento) {
		return MementoHelper.saveMementoToFile(memento, getDataSourceStateFile());
	}	
}
