// $Id$
/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.model;

import java.lang.reflect.InvocationTargetException;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IOperationHistory;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.ILog;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.core.runtime.jobs.ISchedulingRule;
import org.eclipse.jface.action.IStatusLineManager;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.resource.ColorRegistry;
import org.eclipse.jface.resource.FontRegistry;
import org.eclipse.jface.window.ApplicationWindow;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.ide.undo.WorkspaceUndoUtil;
import org.eclipse.ui.part.EditorPart;
import org.eclipse.ui.progress.IProgressService;
import org.eclipse.ui.progress.IWorkbenchSiteProgressService;
import org.eclipse.ui.themes.ITheme;
import org.eclipse.ui.themes.IThemeManager;

import eu.etaxonomy.taxeditor.operation.AbstractPostOperation;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.store.internal.TaxeditorStorePlugin;
import eu.etaxonomy.taxeditor.view.AbstractCdmDataViewer;
import eu.etaxonomy.taxeditor.view.detail.DetailsViewPart;
import eu.etaxonomy.taxeditor.view.supplementaldata.SupplementalDataViewPart;

/**
 * <p>
 * Abstract AbstractUtility class.
 * </p>
 * 
 * @author n.hoffmann
 * @created 11.05.2009
 * @version 1.0
 */
public abstract class AbstractUtility {

	/** Constant <code>statusLineManager</code> */
	protected static IStatusLineManager statusLineManager;

	/**
	 * <p>
	 * closeAll
	 * </p>
	 * 
	 * @return a boolean.
	 */
	public static boolean closeAll() {
		return getActivePage().closeAllEditors(true);
	}

	/**
	 * Close the given editor.
	 * 
	 * @param editor
	 *            The <tt>MultipageTaxonEditor</tt> to close.
	 * @return <tt>true</tt> on success
	 */
	public static boolean close(EditorPart editor) {
		return getActivePage().closeEditor(editor, true);
	}

	/**
	 * <p>
	 * getShell
	 * </p>
	 * 
	 * @return a {@link org.eclipse.swt.widgets.Shell} object.
	 */
	public static Shell getShell() {

		return TaxeditorStorePlugin.getDefault().getWorkbench()
				.getActiveWorkbenchWindow().getShell();
	}

	/**
	 * <p>
	 * getActivePage
	 * </p>
	 * 
	 * @return a {@link org.eclipse.ui.IWorkbenchPage} object.
	 */
	public static IWorkbenchPage getActivePage() {

		return TaxeditorStorePlugin.getDefault().getWorkbench()
				.getActiveWorkbenchWindow().getActivePage();
	}

	/**
	 * <p>
	 * getActivePart
	 * </p>
	 * 
	 * @return a {@link org.eclipse.ui.IWorkbenchPart} object.
	 */
	public static IWorkbenchPart getActivePart() {
		return getActivePage() != null ? getActivePage().getActivePart() : null;
	}

	public static IWorkbench getWorkbench() {
		return TaxeditorStorePlugin.getDefault().getWorkbench();
	}

	/**
	 * <p>
	 * getWorkbenchWindow
	 * </p>
	 * 
	 * @return a {@link org.eclipse.jface.window.ApplicationWindow} object.
	 */
	public static ApplicationWindow getWorkbenchWindow() {
		if (getWorkbench().getWorkbenchWindowCount() > 1) {
			throw new IllegalStateException("More than one workbench window");
		}
		return (ApplicationWindow) getWorkbench().getWorkbenchWindows()[0];
	}

	/**
	 * <p>
	 * showView
	 * </p>
	 * 
	 * @param id
	 *            a {@link java.lang.String} object.
	 * @return a {@link org.eclipse.ui.IViewPart} object.
	 */
	public static IViewPart showView(String id) {
		try {
			return PlatformUI.getWorkbench().getActiveWorkbenchWindow()
					.getActivePage()
					.showView(id, null, IWorkbenchPage.VIEW_VISIBLE);
		} catch (PartInitException e) {
			errorDialog("Error opening view", AbstractUtility.class, "Could not open view: " + id, e);
			return null;
		}
	}

	/**
	 * <p>
	 * hideView
	 * </p>
	 * 
	 * @param view
	 *            a {@link org.eclipse.ui.IViewPart} object.
	 */
	public static void hideView(IViewPart view) {
		PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage()
				.hideView(view);
	}

	/**
	 * <p>
	 * getView
	 * </p>
	 * 
	 * @param id
	 *            a {@link java.lang.String} object.
	 * @param restore
	 *            a boolean.
	 * @return a {@link org.eclipse.ui.IViewPart} object.
	 */
	public static IViewPart getView(String id, boolean restore) {
		IViewReference[] references = PlatformUI.getWorkbench()
				.getActiveWorkbenchWindow().getActivePage().getViewReferences();
		for (IViewReference reference : references) {
			if (reference.getId().equals(id)) {
				return reference.getView(restore);
			}
		}
		return null;
	}

	/**
	 * <p>
	 * getService
	 * </p>
	 * 
	 * @param api
	 *            a {@link java.lang.Class} object.
	 * @return a {@link java.lang.Object} object.
	 */
	public static Object getService(Class api) {
		return TaxeditorStorePlugin.getDefault().getWorkbench().getService(api);
	}

	/**
	 * <p>
	 * getCurrentTheme
	 * </p>
	 * 
	 * @return a {@link org.eclipse.ui.themes.ITheme} object.
	 */
	public static ITheme getCurrentTheme() {
		IThemeManager themeManager = TaxeditorStorePlugin.getDefault()
				.getWorkbench().getThemeManager();
		return themeManager.getCurrentTheme();
	}

	/**
	 * Fonts registered to the plugin may be obtained with the Eclipse themeing
	 * functionality. Thus fonts are chooseable by the user via
	 * Preferences->General->Appearance->Colors and Fonts
	 * 
	 * @return the FontRegistry for the current theme
	 */
	public static FontRegistry getFontRegistry() {
		return getCurrentTheme().getFontRegistry();
	}

	/**
	 * <p>
	 * getFont
	 * </p>
	 * 
	 * @param symbolicName
	 *            a {@link java.lang.String} object.
	 * @return a {@link org.eclipse.swt.graphics.Font} object.
	 */
	public static Font getFont(String symbolicName) {
		return getFontRegistry().get(symbolicName);
	}

	/**
	 * Color registered to the plugin may be obtained with the Eclipse themeing
	 * functionality. Thus colors are editable by the user via
	 * Preferences->General->Appearance->Colors and Fonts
	 * 
	 * @return the ColorRegistry for the current theme
	 */
	public static ColorRegistry getColorRegistry() {
		return getCurrentTheme().getColorRegistry();
	}

	/**
	 * <p>
	 * getColor
	 * </p>
	 * 
	 * @param symbolicName
	 *            a {@link java.lang.String} object.
	 * @return a {@link org.eclipse.swt.graphics.Color} object.
	 */
	public static Color getColor(String symbolicName) {
		return getColorRegistry().get(symbolicName);
	}

	/**
	 * Open a message box that informs the user about unimplemented
	 * functionality. This method is for developer convenience.
	 * 
	 * @param source
	 *            a {@link java.lang.Object} object.
	 */
	public static void notImplementedMessage(Object source) {
		warningDialog("Not yet implemented", source,
				"This functionality is not yet implemented.");
	}

	/**
	 * <p>
	 * informationDialog
	 * </p>
	 * 
	 * @param title
	 *            a {@link java.lang.String} object.
	 * @param message
	 *            a {@link java.lang.String} object.
	 */
	public static void informationDialog(final String title,
			final String message) {
		Display.getDefault().asyncExec(new Runnable() {

			public void run() {
				MessageDialog.openInformation(getShell(), title, message);
			}
		});
	}

	/**
	 * <p>
	 * warningDialog
	 * </p>
	 * 
	 * @param title
	 *            The dialogs title
	 * @param source
	 *            The object where the warning was generated (used by log4j)
	 * @param message
	 *            An informative String to be presented to the user
	 */
	public static void warningDialog(final String title, final Object source,
			final String message) {
		Display.getDefault().asyncExec(new Runnable() {

			public void run() {
				MessageDialog.openWarning(getShell(), title, message);
				Class<? extends Object> clazz = source != null ? source
						.getClass() : AbstractUtility.class;
				warn(clazz, message);
			}
		});
	}

	/**
	 * <p>
	 * errorDialog
	 * </p>
	 * 
	 * @param title
	 *            The dialogs title
	 * @param source
	 *            The object where the warning was generated (used by log4j)
	 * @param message
	 *            An informative String to be presented to the user
	 * @param title
	 *            The dialogs title
	 * @param t
	 *            a Throwable if one exists or null
	 */
	public static void errorDialog(final String title, final Object source,
			final String message, final Throwable t) {
		Display.getDefault().asyncExec(new Runnable() {

			public void run() {
				MessageDialog.openError(getShell(), title, message + getCauseRecursively(t));
				Class<? extends Object> clazz = source != null ? source
						.getClass() : this.getClass();
				error(clazz, message, t);
			}

			private String getCauseRecursively(Throwable t) {
				if(t == null){
					return null;
				}
				
				if(t.getCause() != null){
					return getCauseRecursively(t.getCause());
				}else{
					return String.format("\n\nException: %s\nMessage: %s", t.getClass().getSimpleName(), t.getMessage());
				}

			}
		});
	}

	/**
	 * <p>
	 * errorDialog
	 * </p>
	 * 
	 * @param title
	 *            a {@link java.lang.String} object.
	 * @param source
	 *            a {@link java.lang.Object} object.
	 * @param status
	 *            a {@link org.eclipse.core.runtime.IStatus} object.
	 */
	public static void errorDialog(final String title, final Object source,
			final IStatus status) {
		Display.getDefault().asyncExec(new Runnable() {

			public void run() {
				MessageDialog.openError(getShell(), title, status.getMessage());
				Class<? extends Object> clazz = source != null ? source
						.getClass() : this.getClass();
				error(clazz, status.getMessage(), status.getException());
			}
		});
	}

	/**
	 * <p>
	 * confirmDialog
	 * </p>
	 * 
	 * @param title
	 *            a {@link java.lang.String} object.
	 * @param message
	 *            a {@link java.lang.String} object.
	 * @return a boolean.
	 */
	public static boolean confirmDialog(String title, String message) {
		return MessageDialog.openQuestion(getShell(), title, message);
	}

	/**
	 * <p>
	 * executeOperation
	 * </p>
	 * 
	 * @param operation
	 *            a
	 *            {@link eu.etaxonomy.taxeditor.operation.AbstractPostOperation}
	 *            object.
	 * @return a {@link org.eclipse.core.runtime.IStatus} object.
	 */
	public static IStatus executeOperation(final AbstractPostOperation operation) {
		if (getOperationHistory() == null) {
			throw new IllegalArgumentException(
					"There is no operation history for this context");
		}

		final IAdaptable uiInfoAdapter = WorkspaceUndoUtil
				.getUIInfoAdapter(getShell());

		IRunnableWithProgress runnable = new IRunnableWithProgress() {

			public void run(IProgressMonitor monitor)
					throws InvocationTargetException, InterruptedException {
				monitor.beginTask(operation.getLabel(), 100);
				IStatus status = Status.CANCEL_STATUS;
				try {
					operation.addContext(IOperationHistory.GLOBAL_UNDO_CONTEXT);
					status = getOperationHistory().execute(operation, monitor,
							uiInfoAdapter);
				} catch (ExecutionException e) {
					errorDialog("Error executing operation", getClass(), String.format("An error occured while executing %s.", operation.getLabel()), e);
				} finally {
					monitor.done();
				}
				
				String statusString = status.equals(Status.OK_STATUS) ? "completed"
						: "cancelled";
				setStatusLine(operation.getLabel() + " " + statusString + ".");

			}
		};

		try {
			runInUI(runnable, null);
		} catch (Exception e) {
			errorDialog("Error executing operation", AbstractUtility.class, "An error occured while executing " + operation.getLabel(), e);
		}

		// // Start the main progress monitor.
		// IProgressMonitor newMonitor =
		// startMainMonitor(getMonitor(),operation.getLabel(), 100);
		//
		// // Check whether operation was canceled and do some steps.
		// workedChecked(newMonitor, 10);
		//
		// try {
		// IStatus status = getOperationHistory().execute(operation, newMonitor,
		// WorkspaceUndoUtil.getUIInfoAdapter(getShell()));
		//
		// // Check whether operation was canceled and do some steps.
		// workedChecked(newMonitor, 30);
		//
		// String statusString = status.equals(Status.OK_STATUS) ? "completed" :
		// "cancelled";
		// setStatusLine(operation.getLabel() + " " + statusString + ".");
		//
		// return status;
		// } catch (ExecutionException e) {
		// logger.error("Error executing operation: " + operation.getLabel(),
		// e);
		// errorDialog("Error executing operation: " + operation.getLabel(),
		// "Please refer to the error log.");
		// }
		// finally {
		//
		// // Stop the progress monitor.
		// newMonitor.done();
		// }

		IPostOperationEnabled postOperationEnabled = operation
				.getPostOperationEnabled();
		if (postOperationEnabled != null) {
			postOperationEnabled.onComplete();
		}
		return Status.OK_STATUS;
	}

	/**
	 * <p>
	 * getOperationHistory
	 * </p>
	 * 
	 * @return a {@link org.eclipse.core.commands.operations.IOperationHistory}
	 *         object.
	 */
	public static IOperationHistory getOperationHistory() {
		return getWorkbench().getOperationSupport().getOperationHistory();
	}

	/**
	 * <p>
	 * setStatusLine
	 * </p>
	 * 
	 * @param message
	 *            a {@link java.lang.String} object.
	 */
	public static void setStatusLine(final String message) {
		Display.getDefault().asyncExec(new Runnable() {

			public void run() {
				statusLineManager.setMessage(message);
			}

		});

	}

	/**
	 * <p>
	 * getMonitor
	 * </p>
	 * 
	 * @return a {@link org.eclipse.core.runtime.IProgressMonitor} object.
	 */
	public static IProgressMonitor getMonitor() {
		statusLineManager.setCancelEnabled(false);
		return statusLineManager.getProgressMonitor();
	}

	/**
	 * Starts either the given {@link IProgressMonitor} if it's not
	 * <code>null</code> or a new {@link NullProgressMonitor}.
	 * 
	 * @param progressMonitor
	 *            The {@link IProgressMonitor} or <code>null</code> if no
	 *            progress should be reported.
	 * @param taskName
	 *            The name of the main task.
	 * @param steps
	 *            The number of steps this task is subdivided into.
	 * @return The {@link IProgressMonitor}.
	 */
	public static IProgressMonitor startMainMonitor(
			IProgressMonitor progressMonitor, String taskName, int steps) {
		IProgressMonitor newMonitor = progressMonitor;
		if (newMonitor == null) {
			newMonitor = new NullProgressMonitor();
		}
		newMonitor.beginTask(taskName == null ? "" : taskName, steps);
		newMonitor.subTask(" ");
		return newMonitor;
	}

	/**
	 * Creates a {@link SubProgressMonitor} if the given
	 * {@link IProgressMonitor} is not <code>null</code> and not a
	 * {@link NullProgressMonitor}.
	 * 
	 * @param progressMonitor
	 *            The parent {@link IProgressMonitor} of the
	 *            {@link SubProgressMonitor} to be created.
	 * @param ticks
	 *            The number of steps this subtask is subdivided into. Must be a
	 *            positive number and must not be
	 *            {@link IProgressMonitor#UNKNOWN}.
	 * @return The {@link IProgressMonitor}.
	 */
	public static IProgressMonitor getSubProgressMonitor(
			IProgressMonitor progressMonitor, int ticks) {
		if (progressMonitor == null) {
			return new NullProgressMonitor();
		}
		if (progressMonitor instanceof NullProgressMonitor) {
			return progressMonitor;
		}

		return new SubProgressMonitor(progressMonitor, ticks);
	}

	/**
	 * Checks whether the user canceled this operation. If not canceled, the
	 * given number of steps are declared as done.
	 * 
	 * @param newMonitor
	 *            a {@link org.eclipse.core.runtime.IProgressMonitor} object.
	 * @param steps
	 *            a int.
	 */
	public static void workedChecked(IProgressMonitor newMonitor, int steps) {
		// In case the progress monitor was canceled throw an exception.
		if (newMonitor.isCanceled()) {
			throw new OperationCanceledException();
		}
		// Otherwise declare this step as done.
		newMonitor.worked(steps);
	}

	/**
	 * Present a progress dialog to the user. This dialog will block the UI
	 * 
	 * @param runnable
	 *            an implementation of {@link IRunnableWithProgress}
	 * @throws java.lang.InterruptedException
	 *             if any.
	 * @throws java.lang.reflect.InvocationTargetException
	 *             if any.
	 */
	public static void busyCursorWhile(IRunnableWithProgress runnable)
			throws InvocationTargetException, InterruptedException {
		getProgressService().busyCursorWhile(runnable);
	}

	/**
	 * <p>
	 * runInUI
	 * </p>
	 * 
	 * @see {@link IProgressService#runInUI(org.eclipse.jface.operation.IRunnableContext, IRunnableWithProgress, ISchedulingRule)}
	 * @param runnable
	 *            a {@link org.eclipse.jface.operation.IRunnableWithProgress}
	 *            object.
	 * @param rule
	 *            a {@link org.eclipse.core.runtime.jobs.ISchedulingRule}
	 *            object.
	 * @throws java.lang.reflect.InvocationTargetException
	 *             if any.
	 * @throws java.lang.InterruptedException
	 *             if any.
	 */
	public static void runInUI(IRunnableWithProgress runnable,
			ISchedulingRule rule) throws InvocationTargetException,
			InterruptedException {
		getProgressService().runInUI(getWorkbenchWindow(), runnable, rule);
	}

	/**
	 * <p>
	 * run
	 * </p>
	 * 
	 * @param fork
	 *            a boolean.
	 * @param cancelable
	 *            a boolean.
	 * @param runnable
	 *            a {@link org.eclipse.jface.operation.IRunnableWithProgress}
	 *            object.
	 * @throws java.lang.reflect.InvocationTargetException
	 *             if any.
	 * @throws java.lang.InterruptedException
	 *             if any.
	 */
	public static void run(boolean fork, boolean cancelable,
			IRunnableWithProgress runnable) throws InvocationTargetException,
			InterruptedException {
		getProgressService().run(fork, cancelable, runnable);
	}

	/**
	 * <p>
	 * getProgressService
	 * </p>
	 * 
	 * @return a {@link org.eclipse.ui.progress.IProgressService} object.
	 */
	public static IProgressService getProgressService() {
		IWorkbench workbench = PlatformUI.getWorkbench();
		return workbench.getProgressService();
	}

	/**
	 * <p>
	 * getProgressService2
	 * </p>
	 * 
	 * @return a {@link org.eclipse.ui.progress.IWorkbenchSiteProgressService}
	 *         object.
	 */
	public static IWorkbenchSiteProgressService getProgressService2() {
		return (IWorkbenchSiteProgressService) getService(IWorkbenchSiteProgressService.class);
	}

	/**
	 * <p>
	 * info
	 * </p>
	 * 
	 * @param message
	 *            a {@link java.lang.String} object.
	 */
	public static void info(String message) {
		IStatus status = new Status(IStatus.INFO, getPluginId(), message);
		info(status);
	}

	/**
	 * <p>
	 * info
	 * </p>
	 * 
	 * @param status
	 *            a {@link org.eclipse.core.runtime.IStatus} object.
	 */
	public static void info(IStatus status) {
		log(status);
	}

	/**
	 * <p>
	 * warn
	 * </p>
	 * 
	 * @param source
	 *            a {@link java.lang.Class} object.
	 * @param message
	 *            a {@link java.lang.String} object.
	 */
	public static void warn(Class source, String message) {
		IStatus status = new Status(IStatus.WARNING, getPluginId(), message);
		getLog4JLogger(source).warn(message);
		log(status);
	}
	
	public static void warn(Class source, Throwable t) {
		IStatus status = new Status(IStatus.WARNING, getPluginId(), t.getMessage(), t);
		getLog4JLogger(source).warn(t);
		log(status);
	}

	/**
	 * <p>
	 * error
	 * </p>
	 * 
	 * @param source
	 *            a {@link java.lang.Class} object.
	 * @param t
	 *            a {@link java.lang.Throwable} object.
	 */
	public static void error(Class source, Throwable t) {
		error(source.getClass(), t.getMessage(), t);
	}

	/**
	 * <p>
	 * error
	 * </p>
	 * 
	 * @param source
	 *            a {@link java.lang.Class} object.
	 * @param message
	 *            a {@link java.lang.String} object.
	 * @param t
	 *            a {@link java.lang.Throwable} object.
	 */
	public static void error(Class source, String message, Throwable t) {
		IStatus status = new Status(IStatus.ERROR, getPluginId(), message, t);
		error(source, status);
	}

	/**
	 * <p>
	 * error
	 * </p>
	 * 
	 * @param source
	 *            a {@link java.lang.Class} object.
	 * @param status
	 *            a {@link org.eclipse.core.runtime.IStatus} object.
	 */
	public static void error(Class source, IStatus status) {
		getLog4JLogger(source)
				.error(status.getMessage(), status.getException());
		log(status);
	}

	/**
	 * <p>
	 * getLog4JLogger
	 * </p>
	 * 
	 * @param clazz
	 *            a {@link java.lang.Class} object.
	 * @return a {@link org.apache.log4j.Logger} object.
	 */
	public static Logger getLog4JLogger(Class clazz) {
		return Logger.getLogger(clazz);
	}

	/**
	 * @see {@link ILog#log(IStatus)}
	 * 
	 * @param status
	 */
	private static void log(IStatus status) {
		TaxeditorStorePlugin.getDefault().getLog().log(status);
	}

	/**
	 * <p>
	 * getPluginId
	 * </p>
	 * 
	 * @return a {@link java.lang.String} object.
	 */
	protected static String getPluginId() {
		return "eu.taxeditor";
	}

	/**
	 * <p>
	 * getActiveEditor
	 * </p>
	 * 
	 * @return a {@link org.eclipse.ui.IEditorPart} object.
	 */
	public static IEditorPart getActiveEditor() {
		return getActivePage() != null ? getActivePage().getActiveEditor()
				: null;
	}

	/**
	 * <p>
	 * getDetailsView
	 * </p>
	 * 
	 * @return a {@link eu.etaxonomy.taxeditor.view.detail.DetailsViewPart}
	 *         object.
	 */
	public static DetailsViewPart getDetailsView() {
		return (DetailsViewPart) getView(DetailsViewPart.ID, false);
	}

	/**
	 * <p>
	 * refreshDetailsViewer
	 * </p>
	 */
	public static void refreshDetailsViewer() {
		if (getDetailsView() != null) {
			((AbstractCdmDataViewer) getDetailsView().getViewer()).refresh();
		}
	}

	/**
	 * <p>
	 * reflowDetailsViewer
	 * </p>
	 */
	public static void reflowDetailsViewer() {
		if (getDetailsView() != null) {
			((AbstractCdmDataViewer) getDetailsView().getViewer()).reflow();
		}
	}

	public static SupplementalDataViewPart getSupplementalDataView() {
		return (SupplementalDataViewPart) getView(SupplementalDataViewPart.ID,
				false);
	}

	public static void reflowSupplementalViewer() {
		if (getSupplementalDataView() != null) {
			((AbstractCdmDataViewer) getSupplementalDataView().getViewer())
					.reflow();
		}
	}
}
