// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.store;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import eu.etaxonomy.cdm.api.service.ITermService;
import eu.etaxonomy.cdm.api.service.IVocabularyService;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.common.AnnotationType;
import eu.etaxonomy.cdm.model.common.DefinedTermBase;
import eu.etaxonomy.cdm.model.common.ExtensionType;
import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.cdm.model.common.MarkerType;
import eu.etaxonomy.cdm.model.common.OrderedTermBase;
import eu.etaxonomy.cdm.model.common.OrderedTermVocabulary;
import eu.etaxonomy.cdm.model.common.TermVocabulary;
import eu.etaxonomy.cdm.model.common.VocabularyEnum;
import eu.etaxonomy.cdm.model.description.AbsenceTerm;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.description.MeasurementUnit;
import eu.etaxonomy.cdm.model.description.Modifier;
import eu.etaxonomy.cdm.model.description.PresenceAbsenceTermBase;
import eu.etaxonomy.cdm.model.description.PresenceTerm;
import eu.etaxonomy.cdm.model.description.Scope;
import eu.etaxonomy.cdm.model.description.Stage;
import eu.etaxonomy.cdm.model.description.State;
import eu.etaxonomy.cdm.model.description.StatisticalMeasure;
import eu.etaxonomy.cdm.model.location.NamedAreaType;
import eu.etaxonomy.cdm.model.location.ReferenceSystem;
import eu.etaxonomy.cdm.model.media.RightsTerm;
import eu.etaxonomy.cdm.model.name.NameRelationshipType;
import eu.etaxonomy.cdm.model.name.NameTypeDesignationStatus;
import eu.etaxonomy.cdm.model.name.NomenclaturalStatusType;
import eu.etaxonomy.cdm.model.name.Rank;
import eu.etaxonomy.cdm.model.name.SpecimenTypeDesignationStatus;
import eu.etaxonomy.cdm.model.occurrence.DeterminationModifier;
import eu.etaxonomy.cdm.model.occurrence.PreservationMethod;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationshipType;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;

/**
 * All terms may be accessed through this store.
 * Note: This is for internal use. When using vocabularies in the UI, try to use the preferred terms from
 * PreferenceUtil
 *
 * @author n.hoffmann
 * @created 25.06.2009
 * @version 1.0
 */
public class TermStore {
	
	/**
	 * <p>getSpecimenTypeDesignationStatus</p>
	 *
	 * @return a {@link java.util.List} object.
	 */
	public static List<SpecimenTypeDesignationStatus> getSpecimenTypeDesignationStatus() {
		return getTerms(SpecimenTypeDesignationStatus.class);
	}
	

	/**
	 * <p>getNameTypeDesignationStatus</p>
	 *
	 * @return a {@link java.util.List} object.
	 */
	public static List<NameTypeDesignationStatus> getNameTypeDesignationStatus() {
		return getTerms(NameTypeDesignationStatus.class);
	}
	
	/**
	 * <p>getNamedAreaTypes</p>
	 *
	 * @return a {@link java.util.List} object.
	 */
	public static List<NamedAreaType> getNamedAreaTypes(){
		return getTerms(NamedAreaType.class);
	}
	
	/**
	 * <p>getAnnotationTypes</p>
	 *
	 * @return a {@link java.util.List} object.
	 */
	public static List<AnnotationType> getAnnotationTypes(){
		return getTerms(AnnotationType.class);
	}
	
	/**
	 * <p>getTaxonRelationshipTypes</p>
	 *
	 * @return a {@link java.util.List} object.
	 */
	public static List<TaxonRelationshipType> getTaxonRelationshipTypes(){
		return getTerms(TaxonRelationshipType.class);
	}
	
	
	/**
	 * <p>getLanguages</p>
	 *
	 * @return a {@link java.util.List} object.
	 */
	public static List<Language> getLanguages(){
		return getTerms(Language.class);
	}
	
	/**
	 * <p>getFeatures</p>
	 *
	 * @return a {@link java.util.List} object.
	 */
	public static List<Feature> getFeatures() {
		List<Feature> features = getTerms(Feature.class);
		features.remove(Feature.IMAGE());
		return features;

	}
	
	/**
	 * <p>getNameRelationshipTypes</p>
	 *
	 * @return a {@link java.util.List} object.
	 */
	public static List<NameRelationshipType> getNameRelationshipTypes() {
		return getTerms(NameRelationshipType.class);
	}
	
	/**
	 * <p>getNomenclaturalStatusTypes</p>
	 *
	 * @return a {@link java.util.List} object.
	 */
	public static List<NomenclaturalStatusType> getNomenclaturalStatusTypes(){
		return getTerms(NomenclaturalStatusType.class);
	}
	

	/**
	 * <p>getNonTechnicalMarkerTypes</p>
	 *
	 * @return a {@link java.util.List} object.
	 */
	public static List<MarkerType> getNonTechnicalMarkerTypes() {
		List<MarkerType> nonTechnicalMarkerTypes = new ArrayList<MarkerType>();
		List<MarkerType> markerTypes = getTerms(MarkerType.class);
		
		for (Object type : markerTypes) {
			if (((MarkerType) type).isTechnical() == false) {
				nonTechnicalMarkerTypes.add((MarkerType) type);
			}
		}
		
		return nonTechnicalMarkerTypes;
	}
	

	/**
	 * <p>getExtensionTypes</p>
	 *
	 * @return a {@link java.util.List} object.
	 */
	public static List<ExtensionType> getExtensionTypes() {
		return getTerms(ExtensionType.class);
	}
	
	/**
	 * <p>getRightsTypes</p>
	 *
	 * @return a {@link java.util.List} object.
	 */
	public static List<RightsTerm> getRightsTypes() {
		return getTerms(RightsTerm.class);
	}
	
	
	/**
	 * Retrieve all ranks from data store.
	 *
	 * @return a {@link java.util.List} object.
	 */
	public static List<Rank> getRanks(){
		if(PreferencesUtil.getSortRanksHierarchichally()){
			return getTerms(Rank.class, new Comparator<Rank>(){
	
				@Override
				public int compare(Rank o1, Rank o2) {
					return o1.compareTo(o2);
				}
				
			});
		}else{
			return getTerms(Rank.class);
		}
	}
	
	
	/**
	 * Retrieve all presence and absence terms from data store.
	 *
	 * @return a {@link java.util.List} object.
	 */
	public static List<PresenceAbsenceTermBase> getPresenceAbsenceTerms() {
		List presenceAbsenceTerms = getTerms(PresenceTerm.class);
		presenceAbsenceTerms.addAll(getTerms(AbsenceTerm.class));

		return presenceAbsenceTerms;
	}

	/**
	 * Retrieve all stages from data store.
	 *
	 * @return a {@link java.util.List} object.
	 */
	public static List<Stage> getStages(){
		return getTerms(Stage.class);
	}
	
	/**
	 * Retrieve all states from data store.
	 *
	 * @return a {@link java.util.List} object.
	 */
	public static List<State> getStates() {
		return getTerms(State.class);
	}
	
	/**
	 * Retrieve all preservation methods from data store.
	 *
	 * @return a {@link java.util.List} object.
	 */
	public static List<PreservationMethod> getPreservationMethods(){
		return getTerms(PreservationMethod.class);
	}
	
	

	/**
	 * <p>getMeasurementUnits</p>
	 *
	 * @return a {@link java.util.List} object.
	 */
	public static List<MeasurementUnit> getMeasurementUnits() {
		return getTerms(MeasurementUnit.class);
	}
	

	/**
	 * <p>getModifiers</p>
	 *
	 * @return a {@link java.util.List} object.
	 */
	public static List<Modifier> getModifiers() {
		return getTerms(Modifier.class);
	}


	/**
	 * <p>getStatisticalMeasures</p>
	 *
	 * @return a {@link java.util.List} object.
	 */
	public static List<StatisticalMeasure> getStatisticalMeasures() {
		return getTerms(StatisticalMeasure.class);
	}
	
	/**
	 * <p>getScopes</p>
	 *
	 * @return a {@link java.util.List} object.
	 */
	public static List<Scope> getScopes() {
		return getTerms(Scope.class);
	}
	
	/**
	 * <p>getDeterminationModifiers</p>
	 *
	 * @return a {@link java.util.List} object.
	 */
	public static List<DeterminationModifier> getDeterminationModifiers() {
		return getTerms(DeterminationModifier.class);
	}
	
	/**
	 * <p>getReferenceSystems</p>
	 *
	 * @return a {@link java.util.List} object.
	 */
	public static List<ReferenceSystem> getReferenceSystems() {
		return getTerms(ReferenceSystem.class);
	}
	
	/**
	 ************* save methods *****************************
	 *
	 * @param term a {@link eu.etaxonomy.cdm.model.common.DefinedTermBase} object.
	 */
	public static void saveTerm(DefinedTermBase term){
		getTermService().saveOrUpdate(term);
	}
	
	/**
	 * <p>delete</p>
	 *
	 * @param selected a {@link eu.etaxonomy.cdm.model.common.DefinedTermBase} object.
	 */
	public static void delete(DefinedTermBase selected) {
		getTermService().delete(selected);
	}
	
	/**
	 * Save a vacabulary to data store
	 *
	 * @param term a {@link eu.etaxonomy.cdm.model.common.DefinedTermBase} object.
	 */
	public static void updateVocabulary(DefinedTermBase term) {
		getTermService().saveOrUpdate(term);
	}
	
	
	
	/*************** internal methods **************************/
	
	private static Language getDefaultLanguage(){
		return CdmStore.getDefaultLanguage();
	}
	
	private static OrderedTermVocabulary<? extends OrderedTermBase> getOrderedTermVocabulary(VocabularyEnum vocabularyType){
 		TermVocabulary<DefinedTermBase> vocabulary = getTermVocabulary(vocabularyType);		
 		return HibernateProxyHelper.deproxy(vocabulary, OrderedTermVocabulary.class);
	}
	
	/**
	 * <p>getTermVocabulary</p>
	 *
	 * @param vocabularyType a {@link eu.etaxonomy.cdm.model.common.VocabularyEnum} object.
	 * @return a {@link eu.etaxonomy.cdm.model.common.TermVocabulary} object.
	 */
	public static TermVocabulary<DefinedTermBase> getTermVocabulary(VocabularyEnum vocabularyType){
		
		return CdmStore.getService(IVocabularyService.class).getVocabulary(vocabularyType);
	}
	
	private static <TERM extends DefinedTermBase> List<TERM> getTerms(Class<TERM> clazz){
		Comparator<TERM> comparator = new Comparator<TERM>() {
			@Override
			public int compare(TERM o1, TERM o2) {
				String label1 = o1.getLabel(getDefaultLanguage()) != null ? o1.getLabel(getDefaultLanguage()) : o1.getTitleCache();
				String label2 = o2.getLabel(getDefaultLanguage()) != null ? o2.getLabel(getDefaultLanguage()) : o2.getTitleCache();
				return label1.compareTo(label2);
			}
		};
		
		return getTerms(clazz, comparator);
	}
	
	
	private static <TERM extends DefinedTermBase> List<TERM> getTerms(Class<TERM> clazz, Comparator<TERM> comparator){

		
		List<TERM> terms = new ArrayList<TERM>();
		if(getTermService() != null){
			terms = getTermService().listByTermClass(clazz, null, null, null, null);
		
			Collections.sort(terms, comparator); 
		}
		
		return terms;
		
	}
	
	private static ITermService getTermService(){
		return CdmStore.getService(ITermService.class);
	}
}
