/**
 * 
 */
package eu.etaxonomy.taxeditor.ui.forms;

import java.util.ConcurrentModificationException;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.eclipse.core.runtime.Assert;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.forms.widgets.Section;

import eu.etaxonomy.taxeditor.store.StoreUtil;

/**
 * <p>Abstract AbstractCdmFormElement class.</p>
 *
 * @author n.hoffmann
 * @version $Id: $
 */
public abstract class AbstractCdmFormElement implements IPropertyChangeEmitter, ICdmFormElement, ISelectable {

	protected CdmFormFactory formFactory;

	private List<IPropertyChangeListener> propertyChangeListeners;

	private Composite layoutComposite;
	
	private Set<Control> controls = new HashSet<Control>();
	
	private Set<ICdmFormElement> elements = new HashSet<ICdmFormElement>();
	private ICdmFormElement parentElement;
	
	
	/**
	 * <p>Constructor for AbstractCdmFormElement.</p>
	 *
	 * @param formFactory a {@link eu.etaxonomy.taxeditor.ui.forms.CdmFormFactory} object.
	 * @param layoutComposite a {@link org.eclipse.swt.widgets.Composite} object.
	 */
	protected AbstractCdmFormElement(CdmFormFactory formFactory, Composite layoutComposite){
		this.layoutComposite = layoutComposite;
		this.formFactory = formFactory;
	}
	
	/**
	 * <p>Constructor for AbstractCdmFormElement.</p>
	 *
	 * @param formFactory a {@link eu.etaxonomy.taxeditor.ui.forms.CdmFormFactory} object.
	 * @param formElement a {@link eu.etaxonomy.taxeditor.ui.forms.ICdmFormElement} object.
	 */
	public AbstractCdmFormElement(CdmFormFactory formFactory, ICdmFormElement formElement) {
		this(formFactory, formElement.getLayoutComposite());
		this.parentElement = formElement;
//		addControl(layoutComposite);
	}
	
	/**
	 * <p>Getter for the field <code>formFactory</code>.</p>
	 *
	 * @return a {@link eu.etaxonomy.taxeditor.ui.forms.CdmFormFactory} object.
	 */
	public CdmFormFactory getFormFactory() {
		return formFactory;
	}
	
	/** {@inheritDoc} */
	public Color getColor(boolean selected) {
		return selected ? SELECTED : NOT_SELECTED;
	}
	
	/**
	 * Delegates the focus to <code>this</code> elements main input control
	 */
	public void setFocus(){
		// Override in subclasses where needed 
	}
	
	/**
	 * Returns all Controls that are managed by this element
	 *
	 * @return a {@link java.util.Set} object.
	 */
	public Set<Control> getControls(){
		return controls;
	}
	
	/**
	 * adds the control to the set of controls that are managed by this element
	 *
	 * @param child a {@link org.eclipse.swt.widgets.Control} object.
	 */
	protected void addControl(Control child){
		controls.add(child);
	}
	
	/**
	 * <p>removeControl</p>
	 *
	 * @param child a {@link org.eclipse.swt.widgets.Control} object.
	 */
	protected void removeControl(Control child){
		controls.remove(child);
	}
	
	/**
	 * <p>Getter for the field <code>elements</code>.</p>
	 *
	 * @return a {@link java.util.Set} object.
	 */
	public Set<ICdmFormElement> getElements(){
		return elements;
	}
	
	/**
	 * <p>Getter for the field <code>parentElement</code>.</p>
	 *
	 * @return a {@link eu.etaxonomy.taxeditor.ui.forms.ICdmFormElement} object.
	 */
	public ICdmFormElement getParentElement(){
		return parentElement;
	}
	
	/** {@inheritDoc} */
	public void addElement(ICdmFormElement element){
		elements.add(element);
	}
	
	/**
	 * <p>removeElement</p>
	 *
	 * @param element a {@link eu.etaxonomy.taxeditor.ui.forms.ICdmFormElement} object.
	 */
	protected void removeElement(ICdmFormElement element){
		elements.remove(element);
	}
	
	/**
	 * <p>removeElements</p>
	 */
	public void removeElements(){
		for(ICdmFormElement childElement : getElements()){
			// recursion
			childElement.removeElements();
			
			// unregister selection arbitrator
			if(childElement instanceof ISelectableElement){
				SelectionArbitrator selectionArbitrator = ((ISelectableElement) childElement).getSelectionArbitrator();
				if(selectionArbitrator != null){
					formFactory.destroySelectionArbitrator(selectionArbitrator);
				}
			}
			
			// unregister from property changes
			if(childElement instanceof IPropertyChangeListener){
				formFactory.removePropertyChangeListener((IPropertyChangeListener) childElement); 
			}
			
			// dispose of the controls
			removeControls(childElement);
		}
		removeControls(this);
		elements.clear();
	}
	
	private void removeControls(ICdmFormElement element){
		if(element instanceof Section){
			((Section) element).dispose();
			element = null;
		}else{
			for(Control control : element.getControls()){
				// we added the layoutComposite of the parental element as the layout composite to this formElement
				// but we do not want to destroy it.
				if(control.equals(element.getLayoutComposite())){
					continue;
				}else{
					control.dispose();
					control = null;
				}
			}
		}
	}
	
	/**
	 * <p>Getter for the field <code>layoutComposite</code>.</p>
	 *
	 * @return a {@link org.eclipse.swt.widgets.Composite} object.
	 */
	public Composite getLayoutComposite() {
		return layoutComposite;
	}
	
	/**
	 * <p>Setter for the field <code>layoutComposite</code>.</p>
	 *
	 * @param layoutComposite a {@link org.eclipse.swt.widgets.Composite} object.
	 */
	public void setLayoutComposite(Composite layoutComposite){
		this.layoutComposite = layoutComposite;
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.ICdmFormComposite#getPropertyChangeListeners()
	 */
	/**
	 * <p>Getter for the field <code>propertyChangeListeners</code>.</p>
	 *
	 * @return a {@link java.util.Set} object.
	 */
	public List<IPropertyChangeListener> getPropertyChangeListeners() {
		return propertyChangeListeners;
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.ICdmFormComposite#setPropertyChangeListeners(org.eclipse.core.runtime.ListenerList)
	 */
	/** {@inheritDoc} */
	public void setPropertyChangeListeners(List<IPropertyChangeListener> propertyChangeListeners){
		this.propertyChangeListeners = propertyChangeListeners;
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.IPropertyChangeEmitter#firePropertyChangeEvent()
	 */
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.ICdmFormComposite#firePropertyChangeEvent(org.eclipse.jface.util.PropertyChangeEvent)
	 */
	/** {@inheritDoc} */
	public void firePropertyChangeEvent(CdmPropertyChangeEvent event) {
		Assert.isNotNull(propertyChangeListeners, "Property change listeners are not present");

		try{
			for(Object listener : propertyChangeListeners){
				((IPropertyChangeListener)listener).propertyChange(event);
			}
		}catch(ConcurrentModificationException e){
			// There are two cases that produce a CME. 
			// Described here: http://dev.e-taxonomy.eu/trac/ticket/2363#comment:2
			// and here: http://dev.e-taxonomy.eu/trac/ticket/2438
			// Ignoring the CME because nothing bad is happening
			StoreUtil.warn(getClass(), "ConcurrentModificationException. Can be ignored.");
		}
	}
	
	public void firePropertyChangeEvent(Object object){
		firePropertyChangeEvent(object, null);
	}
	
	public void firePropertyChangeEvent(Object object, PropertyChangeEvent originatingEvent){
		firePropertyChangeEvent(new CdmPropertyChangeEvent(object, originatingEvent));
	}
	

	/**
	 * {@inheritDoc}
	 *
	 * This method gets called whenever the toolkit this composite was created with gets a property change notification.
	 *
	 * It is good advice to check whether the PropertyChangeEvent is destined for the implementing composite.
	 * Implementations should also check for null PropertyChangeEvents and return immediately in that case.
	 * @see eu.etaxonomy.taxeditor.ui.forms.ICdmFormElement#propertyChange(org.eclipse.jface.util.PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent event) {
		// implement in subclasses
	}
	
	/** {@inheritDoc} */
	public boolean containsFormElement(ICdmFormElement formElement){
		if(formElement == this){
			return true;
		}else{
			for(ICdmFormElement element : getElements()){
				boolean contains = element.containsFormElement(formElement);
				if(contains == true){
					return true;
				}
			}
			return false;
		}
	}
	
	/*
	 * (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.ICdmFormElement#refresh()
	 */
	/**
	 * <p>refresh</p>
	 */
	public void refresh() {
		// empty default implementation	
	}
	
	/** {@inheritDoc} */
	@Override
	public void setBackground(Color color) {
		for(ICdmFormElement element : getElements()){
			element.setBackground(color);
		}
	}
	
	public Color getColor(String colorId){
		return StoreUtil.getColor(colorId);
	}
}
