/**
 * 
 */
package eu.etaxonomy.taxeditor.ui.section;

import java.util.Collection;

import org.apache.commons.collections.CollectionUtils;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.ToolBarManager;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.forms.events.ExpansionEvent;
import org.eclipse.ui.forms.events.IExpansionListener;
import org.eclipse.ui.forms.widgets.Section;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.taxeditor.model.ImageResources;
import eu.etaxonomy.taxeditor.preference.IPreferenceKeys;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.preference.Resources;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.ui.forms.AbstractFormSection;
import eu.etaxonomy.taxeditor.ui.forms.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.forms.CdmPropertyChangeEvent;
import eu.etaxonomy.taxeditor.ui.forms.ICdmFormElement;

/**
 * <p>Abstract AbstractEntityCollectionSection class.</p>
 *
 * @author n.hoffmann
 * @version $Id: $
 */
public abstract class AbstractEntityCollectionSection<ENTITY, ELEMENT> extends AbstractFormSection<ENTITY> implements IExpansionListener{
		
	protected Composite container;
	
	private Label label_empty;

	private String title;
	
	/**
	 * <p>Constructor for AbstractEntityCollectionSection.</p>
	 *
	 * @param conversation 
	 * @param parentElement a {@link eu.etaxonomy.taxeditor.ui.forms.ICdmFormElement} object.
	 * @param style a int.
	 * @param formFactory a {@link eu.etaxonomy.taxeditor.ui.forms.CdmFormFactory} object.
	 * @param title a {@link java.lang.String} object.
	 * @param <ENTITY> a ENTITY object.
	 * @param <ELEMENT> a ELEMENT object.
	 */
	public AbstractEntityCollectionSection(CdmFormFactory formFactory, ConversationHolder conversation, ICdmFormElement parentElement, String title, int style) {
		super(formFactory, conversation, parentElement, Section.CLIENT_INDENT | style);
		this.title = title;
		this.setText(getTitleString());
		showToolbar();
		
		addExpansionListener(this);
	}
	
	protected Control createToolbar() {
		ToolBarManager toolBarManager = new ToolBarManager(SWT.FLAT);
		
		Action addAction = new Action("add", Action.AS_PUSH_BUTTON){
			/* (non-Javadoc)
			 * @see org.eclipse.jface.action.Action#run()
			 */
			@Override
			public void run() {
				ELEMENT element = createNewElement();
				if(element != null){
					addElement(element);
					if(! getSection().isExpanded())
						getSection().setExpanded(true);
					internalUpdateSection(true);
				}
			}
		};
		addAction.setImageDescriptor(new ImageDescriptor() {
			
			@Override
			public ImageData getImageData() {
				return ImageResources.getImage(ImageResources.ADD_ICON).getImageData();
			}
		});
		addAction.setToolTipText(getTooltipString());
		
		toolBarManager.add(addAction);
		
		return toolBarManager.createControl(this);
	}
	
	public void showToolbar(){
		setTextClient(createToolbar());
	}
	
	public void removeToolbar(){
		setTextClient(null);
	}
	
	/**
	 * <p>setEntity</p>
	 *
	 * @param entity a ENTITY object.
	 */
	@Override
	public void setEntity(ENTITY entity) {
		if(entity != null && hasCollectionChanged(entity)){
			super.setEntity(entity);
			internalUpdateSection(false);
		}
		setSectionTitle();
		layout();
	};
		
	/**
	 * Sets the title for the section. Adds a "+" sign if the collection is not empty for this section.
	 * Override in subclasses if you want to have a different behaviour.
	 */
	protected void setSectionTitle() {
		if(getCollection(getEntity()) != null && getCollection(getEntity()).size() > 0){
			this.setText(getTitleString() + " +");
		}else{
			this.setText(getTitleString());
		}
	}
	
	/**
	 * Removes all content from the container
	 */
	private void destroyDynamicContent(){
		if(label_empty != null){
			label_empty.dispose();
			label_empty = null;
		}
		removeElements();
	}
	
	/**
	 * Call this method after dynamically changing the client area.
	 * If the options changed is set to true, will also fire a state changed
	 * event to inform the user of unsaved changes.
	 *
	 * @param changed a boolean.
	 */
	protected void internalUpdateSection(boolean changed){
		destroyDynamicContent();
		if(isExpanded() || expandSectionWhenContentAvailable())
			renderContent(isExpanded());
		if(changed)
			firePropertyChangeEvent(new CdmPropertyChangeEvent(this, null));
	}

	/**
	 * Whether the entities specific collection changed
	 * 
	 * @param newEntity
	 * @return
	 */
	private boolean hasCollectionChanged(ENTITY newEntity){

		// return true on null
		if(getEntity() == null || newEntity == null)	return true;	
		
		// if the entities differ the collection has changed
		if(! getEntity().equals(newEntity)) return true;
		
		Collection<ELEMENT> oldCollection = getCollection(getEntity());
		Collection<ELEMENT> newCollection = getCollection(newEntity);
		
		// return true on null
		if(oldCollection == null || newCollection == null) return true;
		
		// if the collections are object equal, check if the content is equal, too
		if(oldCollection.equals(newCollection)){
		
			boolean equal = CollectionUtils.isEqualCollection(oldCollection, newCollection);
			// return true when collections are not equal
			return equal ? false : true;
		}
		return true;
	}

	/**
	 * Create the elements to be shown in this seciton client area 
	 */
	private void renderContent(boolean forceExpansion)
	{
		Collection<ELEMENT> elements = getCollection(getEntity());
		
		if(elements == null || elements.isEmpty()){
			createEmptyContent();
		}else{
			createDynamicContents(elements);
			forceExpansion = true;
		}
		
		this.setExpanded(forceExpansion);
		
		reflow();
	}
	
	/**
	 * <p>createEmptyContent</p>
	 */
	protected void createEmptyContent(){
		label_empty = formFactory.createLabel(getLayoutComposite(), getEmptyString());
	}
	
	/**
	 * Creates the widgets for the collection
	 *
	 * @param elements a {@link java.util.Collection} object.
	 */
	protected void createDynamicContents(Collection<ELEMENT> elements)
	{		
		int i = 0;
		for(final ELEMENT element : elements){
			SelectionAdapter removeListener = new SelectionAdapter(){
				@Override
				public void widgetSelected(SelectionEvent e) {
					removeElement(element);					
					internalUpdateSection(true);
				}
			};
			boolean modulo = i++%2 == 0;
			String colorResource = modulo ? Resources.COLOR_LIST_EVEN : Resources.COLOR_LIST_ODD;
			createElementComposite(element, removeListener, StoreUtil.getColor(colorResource));
		}
	}

	/**
	 * Create the specific widget for the element
	 *
	 * @param element a ELEMENT object.
	 * @param removeListener a {@link org.eclipse.swt.events.SelectionListener} object.
	 * @param backgroundColor a {@link org.eclipse.swt.graphics.Color} object.
	 */
	protected void createElementComposite(ELEMENT element, SelectionListener removeListener, Color backgroundColor){
		AbstractEntityCollectionElement formElement = formFactory.createEntityCollectionElement(this, element, removeListener, backgroundColor, SWT.NULL);
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.section.AbstractEditorFormSection#setBackground(org.eclipse.swt.graphics.Color)
	 */
	/** {@inheritDoc} */
	@Override
	public void setBackground(Color color) {
		if(label_empty != null && !label_empty.isDisposed()){
			label_empty.setBackground(color);
		}
		super.setBackground(color);
	}

	/**
	 * <p>getTitleString</p>
	 *
	 * @return a {@link java.lang.String} object.
	 */
	public String getTitleString() {
		return CdmUtils.Nz(title);
	}
	
	/**
	 * <p>setTitleString</p>
	 *
	 * @param title a {@link java.lang.String} object.
	 */
	public void setTitleString(String title){
		this.title = title;
		setSectionTitle();
		layout();
	}
	
	/** {@inheritDoc} */
	public void expansionStateChanging(ExpansionEvent e) {
//		logger.warn("Expansion State Changing");
	}
	
	/** {@inheritDoc} */
	public void expansionStateChanged(ExpansionEvent e) {
		if(isExpanded()){
			renderContent(isExpanded());
		}else{
			destroyDynamicContent();
		}
	}
	
	private boolean expandSectionWhenContentAvailable(){
		return PreferencesUtil.getPreferenceStore().getBoolean(IPreferenceKeys.SHOULD_EXPAND_SECTION_WHEN_DATA_AVAILABLE);
	}
	
	/**
	 * Get the specific collection of this entity
	 *
	 * @param entity a ENTITY object.
	 * @return a {@link java.util.Collection} object.
	 */
	public abstract Collection<ELEMENT> getCollection(ENTITY entity);
	
	/**
	 * Create a new Element for this collection
	 *
	 * @return a ELEMENT object.
	 */
	public abstract ELEMENT createNewElement();
	
	/**
	 * Add an element to the entities collection
	 *
	 * @param element a ELEMENT object.
	 */
	public abstract void addElement(ELEMENT element);
	
	/**
	 * Remove an element from the entities collection
	 *
	 * @param element a ELEMENT object.
	 */
	public abstract void removeElement(ELEMENT element);
	
	/**
	 * String to display when the collection is empty
	 *
	 * @return a {@link java.lang.String} object.
	 */
	public abstract String getEmptyString();
	
	/**
	 * <p>getTooltipString</p>
	 *
	 * @return String to display when hovering the add button
	 */
	protected abstract String getTooltipString();	
}
