/**
 * 
 */
package eu.etaxonomy.taxeditor.ui.term;


import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;

import eu.etaxonomy.cdm.model.common.DefinedTermBase;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.preference.Resources;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.store.TermManager;
import eu.etaxonomy.taxeditor.ui.forms.AbstractCdmFormElement;
import eu.etaxonomy.taxeditor.ui.forms.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.forms.CdmPropertyChangeEvent;
import eu.etaxonomy.taxeditor.ui.forms.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.forms.IEnableableFormElement;

/**
 * <p>Abstract AbstractTermComboElement class.</p>
 *
 * @author n.hoffmann
 * @created Nov 5, 2009
 * @version 1.0
 * @param <T>
 */
public abstract class AbstractTermComboElement<T extends DefinedTermBase> extends AbstractCdmFormElement implements SelectionListener, DisposeListener, IEnableableFormElement{

	T selection;
	
	private static String EMPTY_ELEMENT_LABEL = "";
	T emptyElement;
	
	protected Label label;
	private Combo combo;

	private ArrayList<T> terms;

	private Comparator<T> termComparator;

	private Composite msgComposite;
	
	/**
	 * <p>Constructor for AbstractTermComboElement.</p>
	 *
	 * @param parentElement a {@link eu.etaxonomy.taxeditor.ui.forms.ICdmFormElement} object.
	 * @param style a int.
	 * @param formFactory a {@link eu.etaxonomy.taxeditor.ui.forms.CdmFormFactory} object.
	 * @param labelString a {@link java.lang.String} object.
	 * @param selection a T object.
	 * @param visibleItems a int.
	 * @param <T> a T object.
	 */
	public AbstractTermComboElement(CdmFormFactory formFactory, ICdmFormElement parentElement, String labelString, T selection, int visibleItems, int style) {
		super(formFactory, parentElement);
		
		if(labelString != null){
			label = formFactory.createLabel(getLayoutComposite(), labelString);
			addControl(label);
		}
		
        // create combo
        combo = new Combo(getLayoutComposite(), SWT.BORDER | SWT.READ_ONLY);
		addControl(combo);
        combo.setLayoutData(CdmFormFactory.FILL_HORIZONTALLY());
        combo.setVisibleItemCount(visibleItems);
        
        populateTerms(preferredTerms());
        
        combo.addSelectionListener(this);
        combo.addDisposeListener(this);
        PreferencesUtil.getPreferenceStore().addPropertyChangeListener(this);
        
        if(selection != null){
        	setSelection(selection);
        }
	}

	/**
	 * <p>Getter for the field <code>selection</code>.</p>
	 *
	 * @return a T object.
	 */
	public T getSelection(){
		return selection;
	}
	
	/**
	 * <p>Setter for the field <code>selection</code>.</p>
	 *
	 * @param selection a T object.
	 */
	public void setSelection(T selection){
		Listener[] listeners = combo.getListeners(SWT.Selection);
		
		for (Listener listener : listeners){
			combo.removeListener(SWT.Selection, listener);
		}
		
		int index = terms.indexOf(selection);
		if(index == -1){
			createTermNotInPreferredTerms(selection);
			index = terms.indexOf(selection);
		}
		combo.select(index);
				
		for (Listener listener : listeners){
			combo.addListener(SWT.Selection, listener);
		}
	}
	
	private void populateTerms(List<T> preferredTerms){
		
		combo.removeAll();
		
		terms = new ArrayList<T>();
		
		int i = 1;
		int index = 0;
		
		// Add an empty element for when nothing was selected already
		combo.add(EMPTY_ELEMENT_LABEL);
		terms.add(emptyElement);
		
		if(termComparator != null){
			Collections.sort(preferredTerms, termComparator);
		}
		for (T term : preferredTerms) {
			String label = getLabel(term); 
			if (label == null) {
				if(term.getTitleCache() != null){
					label = term.getTitleCache();
					StoreUtil.error(getClass(), "Term does not have a representation: " + term + ", " + term.getUuid(), null);
				}else{
					label = "Unknown";
					StoreUtil.error(getClass(), "Representation Label and TitleCache empty for term: " + term + ", " + term.getUuid(), null);
				}
				
			}
							
			combo.add(label);
			terms.add(term);

			i++;
			if (selection != null) {
				if (selection.equals(term)) {
			 		index = i;
				}
			}
		}
		
		if (selection != null && index == 0) {
			createTermNotInPreferredTerms(selection);
		}
		
		combo.select(index);	
	}
	
	/*
	 * (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.IEnableableFormElement#setEnabled(boolean)
	 */
	/** {@inheritDoc} */
	public void setEnabled(boolean enabled) {
		combo.setEnabled(enabled);
	}
	
	/**
	 * <p>preferredTerms</p>
	 *
	 * @return a {@link java.util.List} object.
	 */
	protected abstract List<T> preferredTerms();
	
	/**
	 * May be overridden by derived classes if the desired label string does not
	 * reside in term.getLabel();
	 *
	 * @param term a T object.
	 * @return a {@link java.lang.String} object.
	 */
	protected String getLabel(T term){
		return term.getLabel(CdmStore.getDefaultLanguage());
	}

	private void createTermNotInPreferredTerms(T term) {
		List<T> preferredTerms = preferredTerms();
		
		preferredTerms.add(term);
		
		populateTerms(preferredTerms);
	}
	
	/**
	 * <p>addSelectionListener</p>
	 *
	 * @param listener a {@link org.eclipse.swt.events.SelectionListener} object.
	 */
	public void addSelectionListener(SelectionListener listener){
		combo.addSelectionListener(listener);
	}
	
	/**
	 * <p>removeSelectionListener</p>
	 *
	 * @param listener a {@link org.eclipse.swt.events.SelectionListener} object.
	 */
	public void removeSelectionListener(SelectionListener listener){
		combo.removeSelectionListener(listener);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.swt.events.SelectionListener#widgetSelected(org.eclipse.swt.events.SelectionEvent)
	 */
	/** {@inheritDoc} */
	public void widgetSelected(SelectionEvent e) {
		selection = terms.get(combo.getSelectionIndex());
		firePropertyChangeEvent(new CdmPropertyChangeEvent(this, e));
	}
	
	/* (non-Javadoc)
	 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
	 */
	/** {@inheritDoc} */
	public void propertyChange(PropertyChangeEvent event) {
		super.propertyChange(event);
		if (event != null && PreferencesUtil.PREFERRED_TERMS_CHANGE.equals(event.getProperty())) {
			populateTerms(preferredTerms());
		}
	}
	
	/** {@inheritDoc} */
	public void setSelected(boolean selected) {
		label.setBackground(getColor(selected));
	}
	
	/** {@inheritDoc} */
	public void widgetDisposed(DisposeEvent e) {
		PreferencesUtil.getPreferenceStore().removePropertyChangeListener(this);
	}
	
	// not used
	/** {@inheritDoc} */
	public void widgetDefaultSelected(SelectionEvent e) {}
	
	/** {@inheritDoc} */
	public void setIrrelevant(boolean irrelevant) {
		String colorId = irrelevant ? Resources.COLOR_COMPOSITE_IRRELEVANT : Resources.COLOR_COMPOSITE_BACKGROUND;
		
		Color color = StoreUtil.getColor(colorId);
		combo.setBackground(color);
	}
	
	/** {@inheritDoc} */
	@Override
	public void setBackground(Color color) {
		if(label != null)
			label.setBackground(color);
	}
	
	/**
	 * 
	 */
	protected TermManager getTermManager() {
		return CdmStore.getTermManager();
	}
}
