// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.view;

import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.forms.IFormPart;
import org.eclipse.ui.forms.ManagedForm;
import org.eclipse.ui.forms.widgets.ScrolledForm;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.ui.forms.AbstractFormSection;
import eu.etaxonomy.taxeditor.ui.forms.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.forms.RootElement;
import eu.etaxonomy.taxeditor.view.detail.CdmSectionPart;

/**
 * <p>Abstract AbstractCdmDataViewer class.</p>
 *
 * @author n.hoffmann
 * @created Feb 9, 2010
 * @version 1.0
 */
public abstract class AbstractCdmDataViewer extends Viewer implements ISelectionProvider, IConversationEnabled{
	
	protected ManagedForm managedForm;

	protected CdmFormFactory formFactory;

	protected ScrolledForm scrolledForm;
	
	private Composite body;
	
	protected RootElement rootElement;

	private Object input;

	private AbstractCdmViewPart viewPart;

	
	/**
	 * <p>Constructor for AbstractCdmDataViewer.</p>
	 *
	 * @param parent a {@link org.eclipse.swt.widgets.Composite} object.
	 * @param viewPart a {@link eu.etaxonomy.taxeditor.view.AbstractCdmViewPart} object.
	 */
	public AbstractCdmDataViewer(Composite parent, AbstractCdmViewPart viewPart) {
		this.viewPart = viewPart;
		
		managedForm = new ManagedForm(parent){
			
			/* (non-Javadoc)
			 * @see org.eclipse.ui.forms.ManagedForm#dirtyStateChanged()
			 */
			@Override
			public void dirtyStateChanged() {
				markViewPartDirty();
			}
		};
		
		createFormFactory();
		
		scrolledForm = managedForm.getForm();
		
		body = scrolledForm.getBody();
		
		body.setLayout(CdmFormFactory.LAYOUT());
		
		rootElement = new RootElement(formFactory, body);
	}
	
	/**
	 * <p>createFormFactory</p>
	 */
	protected void createFormFactory() {
		if(formFactory != null){
			formFactory.dispose();
			formFactory = null;
		}
		formFactory = new CdmFormFactory(Display.getCurrent(), this);
	}

	protected void markViewPartDirty(){
		viewPart.changed(input);
	}
	
	
	/**
	 * @return
	 */
	protected AbstractCdmViewPart getViewPart() {
		return viewPart;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.Viewer#getControl()
	 */
	/** {@inheritDoc} */
	@Override
	public Control getControl() {
		for(Control child : body.getChildren()){
			return child;
		}
		
		return body;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.Viewer#setInput(java.lang.Object)
	 */
	/** {@inheritDoc} */
	@Override
	public void setInput(Object input) {
		this.input = input;	
		// reset selection
		setSelection(new StructuredSelection(input));
		refresh();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.Viewer#getInput()
	 */
	/** {@inheritDoc} */
	@Override
	public Object getInput() {
		return input;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.Viewer#refresh()
	 */
	/** {@inheritDoc} */
	@Override
	public void refresh() {
		showParts();
		
		managedForm.setInput(input);
		
		managedForm.refresh();
		
		managedForm.reflow(true);
	}

	
	/**
	 * <p>layout</p>
	 */
	public void layout(){
		body.layout();
	}
	
	/**
	 * <p>showParts</p>
	 */
	protected abstract void showParts();
	
	
	/**
	 * <p>destroySections</p>
	 */
	protected void destroySections() {	
		for (IFormPart formPart : managedForm.getParts()){
			removePart((CdmSectionPart) formPart);
		}	
		
		managedForm.setInput(null);
				
		formFactory.destroyElement(rootElement);
		
		createFormFactory();
		
		rootElement = new RootElement(formFactory, body);
		
		for(Control control : body.getChildren()){
			control.dispose();
			control = null;
		}
	}
	
	
	/**
	 * <p>setFocus</p>
	 */
	public void setFocus(){
		// we have to set focus to a control of this viewer
		// otherwise, after opening a dialog from the details, the focus will not be
		// given back to the details view but to the editor
		for(Control child : body.getChildren()){
			child.setFocus();
			break;
		}
	}
	
	/**
	 * <p>reflow</p>
	 */
	public void reflow(){
		managedForm.reflow(true);
	}
	
	/**
	 * <p>removePart</p>
	 *
	 * @param sectionPart a {@link eu.etaxonomy.taxeditor.view.detail.CdmSectionPart} object.
	 */
	protected void removePart(CdmSectionPart sectionPart){
		managedForm.removePart(sectionPart);
		formFactory.removePropertyChangeListener(sectionPart);
	}
	
	/**
	 * <p>addPart</p>
	 *
	 * @param section a {@link eu.etaxonomy.taxeditor.ui.forms.AbstractFormSection} object.
	 */
	protected void addPart(AbstractFormSection section){
		CdmSectionPart sectionPart = new CdmSectionPart(section);
		managedForm.addPart(sectionPart);
		formFactory.addPropertyChangeListener(sectionPart);
	}
	
	/**
	 * <p>getConversationHolder</p>
	 *
	 * @return a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
	 */
	public ConversationHolder getConversationHolder() {
		return viewPart.getConversationHolder();
	}
	
	/** {@inheritDoc} */
	public void update(CdmDataChangeMap changeEvents) {}

}
